#
#           Yasuharu Okamoto   2020.09
#
import numpy
import statistics
import matplotlib.pyplot as plt
from math import *

class InitCalc:
    """
                   信頼性係数ωとアルファの計算
    """
    def __init__(self, data, f_out):
        self.data = data
        self.f_out = f_out


    def calc_fa(self):
        fout = self.f_out
        data = self.data.T
        n_vars = 0
        var_names = []
        X = []
        for v in data:
            if v[1] == 1:
                var_names.append(v[0])
                X.append(list(v[2:]))
                n_vars += 1        

        n_data = len(X[0])
        print('n_data =', n_data)
        print('n_vars =', n_vars)
        print('var_names = :', var_names)
        fout.write('\nn_data ={}\n'.format(n_data))
        fout.write('n_vars = {}\n'.format(n_vars))
        fout.write('var_names: {}\n'.format(var_names))
        
        X = numpy.array(X)

        sx2 = numpy.cov(X.sum(axis = 0)) 
        
        S = numpy.cov(X)
        
        #
        #       アルファ係数の計算
        #
        gc_alpha = (1.0 - (numpy.diagonal(S).sum() / sx2)) * n_vars / (n_vars - 1)
        print('\nα = {0:.3f}'.format(gc_alpha))
        fout.write('\nα = {0:.3f}\n'.format(gc_alpha))

        #
        #       １因子主因子解
        #
        psi2 = numpy.zeros(n_vars)
        A = numpy.zeros(n_vars)
        while True:
            A0 = A.copy()
            C = S.copy()
            for i in range(n_vars):
                C[i][i] -= psi2[i]
            w, vctr = numpy.linalg.eigh(C)
            A = vctr[:, -1] * (w[-1] ** 0.5)
            for i in range(n_vars):
                psi2[i] = S[i][i] - (A[i] ** 2)
            diff = ((A0 - A) ** 2).sum()
            if diff < 1.0e-7:
                break

        print()
        fout.write('\n')
        for i in range(n_vars):
            print('Lambda[{0}]**2 / Var(X[{0}]) = {1:.3f}'.format(
                    i+1, A[i]**2 / S[i][i]))
            fout.write('Lambda[{0}]**2 / Var(X[{0}]) = {1:.3f}\n'.format(
                    i+1, A[i]**2 / S[i][i]))
        print()
        fout.write('\n')
        for i in range(n_vars):
            print('{0}: Lambda[{1}] = {2:.3f}'.format(
                    var_names[i], i+1, A[i]))
            fout.write('{0}: Lambda[{1}] = {2:.3f}\n'.format(
                    var_names[i], i+1, A[i]))

        #
        #           GFIの計算
        #
        sum_num = 0.0
        sum_den = 0.0
        for i in range(n_vars):
            for j in range(n_vars):
                if i == j:
                    sum_den += S[i][i] ** 2
                else:
                    sum_den += S[i][j] ** 2
                    sum_num += (S[i][j] - A[i] * A[j]) ** 2
        GFI = 1.0 - sum_num / sum_den
        print('\nGFI = {0:.3f}'.format(GFI))
        fout.write('\nGFI = {0:.3f}\n'.format(GFI))
        #
        #           ωの計算
        #       
        omega = (A.sum() ** 2) / sx2
        print('\nω = {0:.3f}'.format(omega))
        fout.write('\nω = {0:.3f}\n'.format(omega))
            
        return 
